#include <GL/glut.h> 
#include <stdlib.h> 
#include <stdio.h>

GLfloat x = 0.0f;
GLfloat y = 0.0f;
GLfloat rsize = 25;

GLfloat xstep = 1.0f;
GLfloat ystep = 1.0f;

GLfloat windowWidth;
GLfloat windowHeight;

static GLfloat xRot = 0.0f;
static GLfloat yRot = 0.0f;
char* filename="model.raw";

#define total_flakes 900
#define total_layers 3
typedef struct{
    int x,y;
    int layer;
} particle;
particle flakes[total_flakes]; 

void initalize_particle_flakes(void)
{
    int i;
    for (i=0;i<total_flakes;i++)
    {
        flakes[i].x=rand()%320; // 0-319 [x]
        flakes[i].y=rand()%200; // 0-199 [y]
        flakes[i].layer=rand()%total_layers;// [0-2] [layer]
    }
}

void update_particle_flakes(void)
{
    int i;
    for (i=0;i<total_flakes;i++)
    {
        // drop the particle down - depending on layer
        // [plus one - since layer zero would result 
        // in no motion otherwize]
        flakes[i].y+=flakes[i].layer+1;
 
        // check for wrap around
        if (flakes[i].y>199)
        {
                flakes[i].x=rand()%320;
                flakes[i].y=0;
                flakes[i].layer=rand()%total_layers;
        }
 
        // new x position        
        flakes[i].x=(flakes[i].x+(2-rand()%5)) % 320;   
    }
}

void RenderScene(void)
{
    int i;
    glClear(GL_COLOR_BUFFER_BIT);
    glColor3f(1.0f, 1.0f, 1.0f);
    glPushMatrix();
        glRotatef(xRot, 1.0f, 0.0f, 0.0f);
        glRotatef(yRot, 0.0f, 1.0f, 0.0f);
        for (i = 0; i < total_flakes; i++){
            GLfloat clr;
            clr = (GLfloat)(flakes[i].layer*5+20) / 30.0f;
            glColor3f(clr, clr, clr);
            glBegin(GL_POINTS);
                glVertex3f((0.5f - (GLfloat)flakes[i].x / 320.0f) * 320.0f, (0.5 - (GLfloat)flakes[i].y / 200.0f) * 200.0f, (GLfloat)(i % 3) * 5.0f - 10.0f);
            glEnd();
        }
        
    glPopMatrix();
    
    glutSwapBuffers();
}

// Called by GLUT library when idle (window not being
// resized or moved)
void TimerFunction(int value)
{
    // Reverse direction when you reach left or right edge
    if(x > windowWidth-rsize || x < -windowWidth)
        xstep = -xstep;

    // Reverse direction when you reach top or bottom edge
    if(y > windowHeight || y < -windowHeight + rsize)
        ystep = -ystep;

    // Actually move the square
    x += xstep;
    y += ystep;

    // Check bounds. This is in case the window is made
    // smaller while the rectangle is bouncing and the 
    // rectangle suddenly finds itself outside the new
    // clipping volume
    if(x > (windowWidth-rsize + xstep)) x = windowWidth-rsize-1;
    else if(x < -(windowWidth + xstep)) x = -windowWidth -1;

    if(y > (windowHeight + ystep)) y = windowHeight-1; 
    else if(y < -(windowHeight - rsize + ystep)) y = -windowHeight + rsize - 1;
    
    update_particle_flakes();
    // Redraw the scene with new coordinates
    glutPostRedisplay();
    glutTimerFunc(33, TimerFunction, 1);
}

void SpecialKeys(int key, int x, int y)
{
    if(key == GLUT_KEY_UP) xRot-= 5.0f;
    if(key == GLUT_KEY_DOWN) xRot += 5.0f;
    if(key == GLUT_KEY_LEFT) yRot -= 5.0f;
    if(key == GLUT_KEY_RIGHT) yRot += 5.0f;
    if(key > 356.0f) xRot = 0.0f;
    if(key < -1.0f) xRot = 355.0f;
    if(key > 356.0f) yRot = 0.0f;
    if(key < -1.0f) yRot = 355.0f;

    // Refresh the Window
    glutPostRedisplay();
}

// Setup the rendering state
void SetupRC(void)
{
    // Set clear color to blue
    glClearColor(0.0f, 0.0f, 0.0f, 1.0f);
    initalize_particle_flakes();
}

void ChangeSize(int w, int h)
{
    GLfloat nRange = 100.0f;

    // Prevent a divide by zero
    if(h == 0) h = 1;

    // Set Viewport to window dimensions
    glViewport(0, 0, w, h);

    // Reset projection matrix stack
    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();

    // Establish clipping volume (left, right, bottom, top, near, far)
    if (w <= h)  glOrtho (-nRange, nRange, -nRange*h/w, nRange*h/w, -nRange, nRange);
    else  glOrtho (-nRange*w/h, nRange*w/h, -nRange, nRange, -nRange, nRange);

    // Reset Model view matrix stack
    glMatrixMode(GL_MODELVIEW);
    glLoadIdentity();
}

// Main program entry point
int main(int argc, char* argv[])
{
    glutInit(&argc, argv);
    glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGBA | GLUT_DEPTH);
    glutInitWindowSize(320,200);
    glutCreateWindow("Snow");
    glutReshapeFunc(ChangeSize);
    glutDisplayFunc(RenderScene);
    glutSpecialFunc(SpecialKeys);
    glutTimerFunc(33, TimerFunction, 1);

    SetupRC();

    glutMainLoop();

    return 0;
}
